﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using XrMapLib;

namespace HilightingShape
{
    class DuraMapUtility_HilightingShape
    {
        private AxXrMapLib.AxXr XrMap = null;
        private Boolean bDrawHilghting = false;
        private Boolean bToggle = true;
        private ArrayList hilightPtsGroup = null;
        private XrMapLib.XrShapeTypeEnum hilightShpType;

        public DuraMapUtility_HilightingShape(AxXrMapLib.AxXr XrMap)
        {
            this.XrMap = XrMap;
        }

        public void Hilighting(String LayerName, int FID)
        {
            ShapeMapLayer sl = XrMap.Layers.GetLayerAsShapeMap(LayerName);
            if (sl == null) return;

            hilightShpType = sl.ShapeTable.ShapeType;

            XrMapLib.ShapeRow row = sl.ShapeTable.GetRow(FID);
            if (row == null) return;
            if (row.Load())
            {
                XrShapeTypeEnum st = sl.ShapeTable.ShapeType;
                int cntParts = row.PartCount;
                hilightPtsGroup = new ArrayList(cntParts);

                if (st == XrShapeTypeEnum.XrPolyline || st == XrShapeTypeEnum.XrPolygon)
                {
                    for (int iPart = 0; iPart < cntParts; ++iPart)
                    {
                        int cntVertex = row.GetVertexCountFromPart(iPart);
                        Point[] pts = new Point[cntVertex];
                        hilightPtsGroup.Add(pts);
                        for (int iVertex = 0; iVertex < cntVertex; ++iVertex)
                        {
                            XrMapLib.Coord coord = row.GetVertexFromPart(iPart, iVertex);
                            Coord vp = XrMap.MapCoordToScreen(coord.X, coord.Y);
                            pts[iVertex].X = (int)vp.X;
                            pts[iVertex].Y = (int)vp.Y;
                        }
                    }
                }
                else
                {
                    int cntVertex = row.GetVertexCountFromPart(0);
                    Point[] pts = new Point[cntVertex];
                    hilightPtsGroup.Add(pts);
                    for (int iVertex = 0; iVertex < cntVertex; ++iVertex)
                    {
                        XrMapLib.Coord coord = row.GetVertex(iVertex);
                        Coord vp = XrMap.MapCoordToScreen(coord.X, coord.Y);
                        pts[iVertex].X = (int)vp.X;
                        pts[iVertex].Y = (int)vp.Y;
                    }
                }

                DoHilightingShape();

                row.Unload();
            }
        }

        private void DoHilightingShape()
        {
            bDrawHilghting = true;

            Thread.Sleep(700);

            bDrawHilghting = false;
        }

        public void OnCustomDrawEvent(AxXrMapLib._IXrMapControlEvents_OnCustomDrawEvent e)
        {
            if (bDrawHilghting && hilightPtsGroup != null)
            {
                IntPtr hDC = (IntPtr)e.hDC;

                Color clr = Color.FromArgb(220, 0, 255, 255);
                Pen pen = new Pen(clr);

                if (bToggle) pen.Width = 6;
                else pen.Width = 3;

                SolidBrush brush = new SolidBrush(clr);
                if (bToggle) brush.Color = clr;
                else brush.Color = Color.FromArgb(100, 0, 255, 255);

                Graphics gfx = Graphics.FromHdc(hDC);

                if (hilightShpType == XrShapeTypeEnum.XrPolygon)
                {

                    for (int i = 0; i < hilightPtsGroup.Count; i++)
                    {
                        Point[] pts = (Point[])(hilightPtsGroup[i]);
                        if (pts.Length > 2)
                        {
                            gfx.FillPolygon(brush, pts);
                            gfx.DrawPolygon(pen, pts);
                        }
                    }
                }
                else if (hilightShpType == XrShapeTypeEnum.XrPolyline)
                {
                    for (int i = 0; i < hilightPtsGroup.Count; i++)
                    {
                        Point[] pts = (Point[])(hilightPtsGroup[i]);
                        gfx.DrawLines(pen, pts);
                    }
                }
                else
                {
                    for (int i = 0; i < hilightPtsGroup.Count; i++)
                    {
                        Point[] pts = (Point[])(hilightPtsGroup[i]);
                        for (int j = 0; j < pts.Length; j++)
                        {
                            gfx.DrawEllipse(pen, pts[j].X - 4, pts[j].Y - 4, 10, 10);
                        }
                    }
                }

                //gfx.ReleaseHdc(hDC);
                gfx.Dispose();

                bToggle = !bToggle;
            }
        }
    }
}